from typing import Any

from cfgs import App


_config_file = App("alga").config.open("config.json")


def get() -> dict[str, Any]:
    config = _config_file.contents
    config.setdefault("version", 1)
    return config


def write(config: dict[str, Any]) -> None:
    _config_file.contents = config
    _config_file.write()



import ssl
from collections.abc import Iterator
from contextlib import contextmanager
from typing import Annotated

from typer import Argument, Typer
from websocket import WebSocket

from alga import client


app = Typer(no_args_is_help=True, help="Remote control button presses")


@contextmanager
def _input_connection() -> Iterator[WebSocket]:  # pragma: no cover
    response = client.request(
        "ssap://com.webos.service.networkinput/getPointerInputSocket"
    )

    connection = WebSocket(sslopt={"cert_reqs": ssl.CERT_NONE})
    connection.connect(response["socketPath"], suppress_origin=True, timeout=3)  # type: ignore[no-untyped-call]

    try:
        yield connection
    finally:
        connection.close()


@app.command()
def send(button: Annotated[str, Argument()]) -> None:
    """Send a button press to the TV"""
    with _input_connection() as connection:
        connection.send(f"type:button\nname:{button.upper()}\n\n")



from typer import Typer
from wakeonlan import send_magic_packet

from alga import client, config


app = Typer(no_args_is_help=True, help="Turn TV (or screen) on and off")


@app.command()
def off() -> None:
    """Turn TV off"""

    client.request("ssap://system/turnOff")


@app.command()
def on() -> None:
    """Turn TV on via Wake-on-LAN"""

    cfg = config.get()
    send_magic_packet(cfg["mac"])


@app.command()
def screen_off() -> None:
    """Turn TV screen off"""

    client.request("ssap://com.webos.service.tvpower/power/turnOffScreen")


@app.command()
def screen_on() -> None:
    """Turn TV screen on"""

    client.request("ssap://com.webos.service.tvpower/power/turnOnScreen")



import json
from ipaddress import ip_address
from socket import gaierror, getaddrinfo
from typing import Annotated, Optional

from getmac import get_mac_address
from rich import print
from rich.console import Console
from typer import Argument, Exit

from alga import client, config
from alga.payloads import get_hello_data


def _ip_from_hostname(hostname: str) -> Optional[str]:
    try:
        results = getaddrinfo(host=hostname, port=None)
        # TODO: Do we want to handle receiving multiple IP addresses?
        return results[0][4][0]
    except gaierror:
        return None


def setup(
    hostname: Annotated[str, Argument()] = "lgwebostv",
) -> None:  # pragma: no cover
    """Pair a new TV"""

    # Check if we have been passed an IP address
    ip: Optional[str]
    try:
        ip = ip_address(hostname).compressed
    except ValueError:
        ip = _ip_from_hostname(hostname)
        if not ip:
            print(
                f"[red]Could not find any host by the name '{hostname}'.[/red] Is the TV on and connected to the network?"
            )
            raise Exit(code=1)

    with client.new(
        hostname=hostname, perform_handshake=False, timeout=60
    ) as connection:
        connection.send(json.dumps(get_hello_data()))
        response = json.loads(connection.recv())
        assert response == {
            "id": "register_0",
            "payload": {"pairingType": "PROMPT", "returnValue": True},
            "type": "response",
        }, "Unexpected response received"

        console = Console()
        with console.status("Please approve the connection request on the TV now..."):
            response = json.loads(connection.recv())

    if "client-key" not in response["payload"]:
        print("[red]Setup failed![/red]")
        raise Exit(code=1)

    mac_address = get_mac_address(ip=ip)

    cfg = config.get()
    cfg.update(
        hostname=hostname, key=response["payload"]["client-key"], mac=mac_address
    )
    config.write(cfg)

    print("TV configured, Alga is ready to use")



from rich import print
from typer import Typer

from alga import (
    __version__,
    cli_adhoc,
    cli_app,
    cli_channel,
    cli_input,
    cli_media,
    cli_power,
    cli_remote,
    cli_setup,
    cli_sound_output,
    cli_volume,
)


app = Typer(no_args_is_help=True)
app.add_typer(cli_app.app, name="app")
app.add_typer(cli_channel.app, name="channel")
app.add_typer(cli_input.app, name="input")
app.add_typer(cli_media.app, name="media")
app.add_typer(cli_power.app, name="power")
app.add_typer(cli_remote.app, name="remote")
app.add_typer(cli_sound_output.app, name="sound-output")
app.add_typer(cli_volume.app, name="volume")

# Commands that only has a single sub-command
# https://github.com/tiangolo/typer/issues/243
app.command()(cli_adhoc.adhoc)
app.command()(cli_setup.setup)


@app.command()
def version() -> None:
    """Print Alga version"""

    print(f"alga version [bold]{__version__}[/bold]")


if __name__ == "__main__":
    app()



import json
from typing import Annotated, Optional

from rich import print
from rich.console import Console
from rich.table import Table
from typer import Argument, Typer

from alga import client


app = Typer(no_args_is_help=True, help="Apps installed on the TV")


@app.command()
def current() -> None:
    """Get the current app"""

    response = client.request(
        "ssap://com.webos.applicationManager/getForegroundAppInfo"
    )
    print(f"The current app is [bold]{response['appId']}[/bold]")


@app.command()
def close(app_id: Annotated[str, Argument()]) -> None:
    """Close the provided app"""

    client.request("ssap://system.launcher/close", {"id": app_id})


@app.command()
def launch(
    app_id: Annotated[str, Argument()],
    data: Annotated[Optional[str], Argument()] = None,
) -> None:
    """Launch an app"""

    payload = {"id": app_id}
    if data:
        payload.update(json.loads(data))
    client.request("ssap://system.launcher/launch", payload)


@app.command()
def list() -> None:
    """List installed apps"""

    response = client.request("ssap://com.webos.applicationManager/listApps")

    table = Table()
    table.add_column("Name")
    table.add_column("ID")

    all_apps = []
    for app in response["apps"]:
        all_apps.append([app["title"], app["id"]])

    for row in sorted(all_apps):
        table.add_row(*row)

    console = Console()
    console.print(table)


@app.command()
def info(app_id: str) -> None:
    """Show info about specific app"""

    response = client.request(
        "ssap://com.webos.applicationManager/getAppInfo", {"id": app_id}
    )

    print(response["appInfo"])



import json
import ssl
from collections.abc import Iterator
from contextlib import contextmanager
from typing import Any, Optional, cast

from rich import print
from typer import Exit
from websocket import WebSocket

from alga import config
from alga.payloads import get_hello_data


@contextmanager
def new(
    hostname: Optional[str] = None, perform_handshake: bool = True, timeout: int = 3
) -> Iterator[WebSocket]:  # pragma: no cover
    cfg = config.get()

    if hostname is None:
        if "hostname" not in cfg:
            print("[red]No connection configured, run 'alga setup' first[/red]")
            raise Exit(code=1)

        hostname = cfg["hostname"]

    connection = WebSocket(sslopt={"cert_reqs": ssl.CERT_NONE})
    connection.connect(f"wss://{hostname}:3001/", suppress_origin=True, timeout=timeout)  # type: ignore[no-untyped-call]

    if perform_handshake:
        if "key" not in cfg:
            print("[red]No connection configured, run 'alga setup' first[/red]")
            raise Exit(code=1)

        connection.send(json.dumps(get_hello_data(cfg["key"])))
        response = json.loads(connection.recv())
        if "client-key" not in response["payload"]:
            raise Exception(
                f"Something went wrong with performing a handshake. Response: {response}"
            )

    try:
        yield connection
    finally:
        connection.close()


def request(
    uri: str, data: Optional[dict[str, Any]] = None
) -> dict[str, Any]:  # pragma: no cover
    with new() as connection:
        request: dict[str, Any] = {"type": "request", "uri": uri}

        if data:
            request.update(payload=data)

        connection.send(json.dumps(request))

        raw_response = connection.recv()
        response = json.loads(raw_response)

        assert response.get("payload", {}).get("returnValue") is True
        return cast(dict[str, Any], response["payload"])



from typing import Annotated

from rich import print
from typer import Argument, Typer

from alga import client


app = Typer(no_args_is_help=True, help="Audio output device")


@app.command()
def get() -> None:
    """Show the current output device"""

    response = client.request("ssap://audio/getSoundOutput")
    print(f"The current sound output is [bold]{response['soundOutput']}[/bold]")


@app.command()
def set(value: Annotated[str, Argument()]) -> None:
    """Change the output device"""

    client.request("ssap://audio/changeSoundOutput", {"output": value})
